﻿using System;

namespace C4Tools.ML2
{
    /// <summary>
    /// Provides an interface for WPF to display the progress of an operation
    /// </summary>
    public interface IProgressWorker
    {
        /// <summary>
        /// Gets the progress total (the number that represents the progress end)
        /// </summary>
        int Total { get; }

        /// <summary>
        /// The current progress value of the current stage
        /// </summary>
        int Current { get; }

        /// <summary>
        /// The current stage in operation
        /// </summary>
        String Stage { get; }

        /// <summary>
        /// Starts the progress operation in a BackgroundWorker thread
        /// </summary>
        void Start();

        /// <summary>
        /// Starts the progress operation, possibly in a BackgroundWorker thread
        /// </summary>
        void Start(bool wantSta);

        /// <summary>
        /// Requests cancellation of the progress operation
        /// </summary>
        void CancelAsync();

        /// <summary>
        /// Occurs when the stage of the operation has changed
        /// </summary>
        event EventHandler StageChanged;

        /// <summary>
        /// Occurs when the progress of the operation has changed
        /// </summary>
        event EventHandler ProgressChanged;

        /// <summary>
        /// Occurs when the total of the progress operation has changed (the number that represents the progress end)
        /// </summary>
        event EventHandler ProgressTotalChanged;

        /// <summary>
        /// Occurs when the progress operation is complete
        /// </summary>
        event EventHandler Complete;
    }
}
